using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using HPS;

namespace hps_cs_samples
{
    public partial class Sample
    {
        public void Interpolation(WindowKey wk, Canvas canvas, View view, Model model, SegmentKey modelKey)
        {
            /* Sample: Interpolation
	        * This sample will show you how to:
	        * - Create a material palette
	        * - Set colors on vertices using an RGBColor
	        * - Set colors on vertices using a material palette
	        * - Turning Color Interpolation ON and OFF
	        * 
	        * Read more about the topics covered in this sample in our Programming Guide 
	        * - section 4.3:	Color
	        * - section 4.2:	Materials and Material Palettes */

            modelKey.GetMaterialMappingControl().SetFaceColor(new HPS.RGBAColor(0, 0, 1, 1));

	        const int numberOfSides = 36;
	        const int numberOfVertices = 2 * (numberOfSides + 1);
            const double PI = 3.141592653589793238462643383279502884197169399375105820974944592308;

	        float[] radii = new float[4];
	        radii[0] = 0.5f;
            radii[1] = 1;
            radii[2] = 0.5f;
            radii[3] = 0.8f;

            float[] yValue = new float[2];
	        yValue[0] = 0.5f;
            yValue[1] = 0;

	        int vertex;
	        HPS.Point[] meshPoints = new HPS.Point[numberOfVertices];
	        float angle;

	        for (vertex = 0; vertex < numberOfVertices; ++vertex) 
	        {
		        angle = (float)(vertex / 2.0f) / (float)numberOfSides * (2 * (float)PI);
		        meshPoints[vertex].x = radii[vertex % 4] * (float)Math.Cos(angle);
		        meshPoints[vertex].y = yValue[vertex % 2];
		        meshPoints[vertex].z = radii[vertex % 4] * (float)Math.Sin(angle);
	        }

	        HPS.SegmentKey geometry = modelKey.Subsegment();
	        geometry.GetVisibilityControl().SetEverything(false);
            HPS.MeshKey mesh = geometry.InsertMesh(numberOfSides + 1, 2, meshPoints);
	        for (vertex = 0; vertex < numberOfVertices; ++vertex) 
	        {
		        mesh.SetVertexIndexColorsByRange((ulong)vertex, 1, (float)(vertex % 2) * 3);
	        }

            HPS.TextKit textKit = new HPS.TextKit();
            textKit.SetSize(0.02f, HPS.Text.SizeUnits.WindowRelative);
            textKit.SetPosition(new HPS.Point(0, -0.2f, 0));
            textKit.SetAlignment(HPS.Text.Alignment.CenterRight);

	        /* This next block of code shows how to define a Material Palette
	         * 1. Material Palettes need to be defined in a portfolio, so the first step is to
	         *    obtain a PortfolioKey object.
	         * 2. Material Palettes, when set on a segment, inherit down the scene graph like 
	         *    attributes. Therefore it is safe to set the material palette on the modelKey
	         *    segment: the box segment will inherit this material palette
	         * 3. Three MaterialKit objects are created. In this example each material kit describes
	         *    a simple diffuse color, but many different attributes can be set on a single kit
	         * 4. A MaterialPalette is defined in a portfolio by supplying a name for the material palette
	         *    and an array of MaterialKit objects
	         * 5. The material palette is set on the segment. 
	         *    At this point the materials defined in the myPalette palette can be used on this segment
	         * */
            HPS.MaterialKit[] materialKit = new MaterialKit[4];
            HPS.PortfolioKey portfolio;
	        modelKey.GetPortfolioControl().ShowTop(out portfolio);
            materialKit[0] = new HPS.MaterialKit().SetDiffuse(new HPS.RGBAColor(1, 0, 0, 1));
            materialKit[1] = new HPS.MaterialKit().SetDiffuse(new HPS.RGBAColor(0, 1, 0, 1));
            materialKit[2] = new HPS.MaterialKit().SetDiffuse(new HPS.RGBAColor(0, 0, 1, 1));
            materialKit[3] = new HPS.MaterialKit().SetDiffuse(new HPS.RGBAColor(1, 1, 0, 1));
            HPS.MaterialPaletteDefinition mpd = portfolio.DefineMaterialPalette("myPalette", materialKit);
	        modelKey.SetMaterialPalette("myPalette");

	        /* This block of code shows how to color a shell using color interpolation
	         * 1. Insert the shell
	         * 2. Set a color on its vertices, it is possible to set the same color on many vertices at once 
	         * 3. Turn color interpolation ON for faces on using a ColorInterpolationControl */
            HPS.SegmentKey s1 = modelKey.Subsegment();
            s1.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(-1, 0, 0, 1), HPS.Subwindow.Type.Lightweight);
            s1.ReferenceGeometry(mesh);
            s1.GetColorInterpolationControl().SetFaceColor(true).SetFaceIndex(false);
	        textKit.SetText("Color \nInterpolation");
	        s1.InsertText(textKit);

	        /* This block of code shows how to color a shell without using color interpolation
	         * 1. Insert the shell
	         * 2. Set a color on its vertices, it is possible to set the same color on many vertices at once 
	         * 3. Turn color interpolation OFF for faces on using a ColorInterpolationControl 
	         * Since color interpolation is off, the face color declared above will be used instead of the vertices colors
	         * for faces */
            HPS.SegmentKey s2 = modelKey.Subsegment();
            s2.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(0, 1, 0, 1), HPS.Subwindow.Type.Lightweight);
            s2.ReferenceGeometry(mesh);
            s2.GetColorInterpolationControl().SetFaceColor(false).SetFaceIndex(false);
	        textKit.SetText("No Color \nInterpolation");
	        s2.InsertText(textKit);

	        /* This block of code shows how to color a shell by index
	         * 1. Insert the shell
	         * 2. Set a color on its vertices by index. The indices used refer to the MaterialKits in the
	         *    MaterialPalette which is currently active on this segment. The MaterialPalette has been defined
	         *    and set as active above.
	         * 3. Turn color interpolation ON for faces on using a ColorInterpolationControl */
            HPS.SegmentKey s3 = modelKey.Subsegment();
            s3.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(-1, 0, -1, 0), HPS.Subwindow.Type.Lightweight);
            s3.ReferenceGeometry(mesh);
            s3.GetColorInterpolationControl().SetFaceColor(true).SetFaceIndex(true);
	        textKit.SetText("Color Interpolation & \nColor Index Interpolation");
	        s3.InsertText(textKit);

	        /* This block of code shows how to color a shell by index, without using color interpolation
	         * 1. Insert the shell
	         * 2. Set a color on its vertices by index. The indices used refer to the MaterialKits in the
	         *    MaterialPalette which is currently active on this segment. The MaterialPalette has been defined
	         *    and set as active above.
	         * 3. Turn color interpolation OFF for faces on using a ColorInterpolationControl */
            HPS.SegmentKey s4 = modelKey.Subsegment();
            s4.GetSubwindowControl().SetSubwindow(new HPS.Rectangle(0, 1, -1, 0), HPS.Subwindow.Type.Lightweight);
            s4.ReferenceGeometry(mesh);
            s4.GetColorInterpolationControl().SetFaceColor(false).SetFaceIndex(true);
	        textKit.SetText("No Color Interpolation & \nColor Index Interpolation");
	        s4.InsertText(textKit);

	        view.FitWorld();
            
        }
    }
}
